import numpy as np
class LinearRegressor:
    '''Linear Regressor instance'''
    def __init__(self):
        pass

    def _compute(self, x, y):
        try:
            var = np.dot(x.T,x)
            var = np.linalg.inv(var)
            var = np.dot(var,x.T)
            var = np.dot(var,y)
            self.__thetas = var
        except Exception as e:
            raise e
    
    def fit(self, X, y):
        '''
        Trains the `LinearRegressor` instance, returns none

        Parameters:
        -----------
        `X`: a ndarray for the training input
        `y`: a ndarray for the training output

        Returns: None
        '''
        x = np.array(X)
        ones_ = np.ones(x.shape[0])
        x = np.c_[ones_,x]
        y = np.array(y)
        self._compute(x,y)
    
    def predict(self, X):
        '''
        predicts `y` value for the `x` input

        Parameter: `X`: a ndarray of shape (num_samples, num_features) to be evaluated

        Returns: `y`: float for the predicted value
        '''
        x = np.array(X)
        ones_ = np.ones(x.shape[0])
        x = np.c_[ones_,x]
        result = np.dot(x,self.__thetas)
        return result  

class SingLinearRegressor:
    '''
    Linear Regressor for `x` being a single value (as opposed to an ndarray)

    Slightly faster than `LinearRegressor`, good for large amounts of data

    '''
    def __init__(self):
        self.bet0: float
        self.bet1: float

    def fit(self, X, y: np.array) -> None:
        '''
        Trains the `LinearRegressor` instance, returns none

        Parameters:
        -----------
        `X`: a ndarray 
        `y`: a ndarray

        Returns: None
        '''
        n = np.size(X)

        meanX = np.mean(X)
        meany = np.mean(y)

        cc_xy = np.sum(y*X) - n*meany*meanX
        cc_xx = np.sum(X*X) - n*meanX*meanX

        bet1 = cc_xy / cc_xx
        bet0 = meany - bet1*meanX

        self.bet0 = bet0
        self.bet1 = bet1

    def predict(self, x: float) -> float:
        '''
        Predicts `y` value for instance `x`

        Parameter:
        ----------
        `x`: float representing single data instance

        Returns:
        --------
        `y`: float representing `y` value for `x` instance
        '''
        pred = self.bet0 + self.bet1*x
        return pred