from regression import LinearRegressor
import numpy as np
from itertools import combinations, combinations_with_replacement

class PolyRegressor:
    """Polynomial Regressor instance, uses the degree of the data"""
    def __init__(self, degree=2):
        self.degree = degree
    
    lin = LinearRegressor()

    def evalnum(self, X):
        """Converts ndarray to poly features"""
        ret = []
        for j in range(len(X)):
            l = np.ndarray.tolist(X[j])
            l.append("c")
            t = []
            for i in combinations_with_replacement(l, self.degree):
                t.append(i)
            t = [list(_) for _ in t]
            for i in range(len(t)):
                while "c" in t[i]:
                    t[i].remove("c")
                t[i] = np.prod(t[i])
            t = np.array(t)
            t = np.sort(t)
            ret.append(t)
        ret = np.array(ret)
        ret = np.sort(ret, 1)
        return ret

    def fit(self, X, y):
        """
        Fits the `PolyRegressor` instance

        Parameters:
        ===========
        `X`: ndarray of shape (num_samples, num_features) for input
        `y`: numpy array of shape (num_samples,) for output

        returns none
        """
        X = self.evalnum(X)
        self.lin.fit(X, y)

    def predict(self, X):
        """
        Predicts `y` value for `x` instance

        Parameter: `X`: ndarray of shape (num_samples, num_features)

        Returns: `y`: ndarray containning `y` value
        """
        X = self.evalnum(X)
        return self.lin.predict(X)

